@@ -65,11 +65,12 @@ class NSO_JSON_Response:
 		return self.payload.get('result')
 
 class NSO_API:
-	FALLBACK_APP_VERSION = "2.4.0"
+	FALLBACK_APP_VERSION = "2.5.0"
 
 	global_data = {}
+	global_callbacks = {}
 
-	def __init__(self, f_provider, context = None):
+	def __init__(self, f_provider, context = None, debug = 0):
 		self.session = requests.Session()
 		self.f_provider = f_provider
 		self.client_id = '71b963c1b7b6d119'
@@ -87,7 +88,7 @@ class NSO_API:
 		self.s3 = NSO_API_S3(self)
 		self.acnh = NSO_API_ACNH(self)
 		self.account = NSO_API_Account(self)
-		self.debug = int(os.environ.get('NSO_API_DEBUG', 0))
+		self.debug = max(int(os.environ.get('NSO_API_DEBUG', 0)), debug)
 		self.errors = []
 
 	@classmethod
@@ -127,8 +128,9 @@ class NSO_API:
 	def on_user_data_update(self, callback):
 		self.callbacks['user_data_update'] = callback
 
-	def on_global_data_update(self, callback):
-		self.callbacks['global_data_update'] = callback
+	@classmethod
+	def on_global_data_update(cls, callback):
+		cls.global_callbacks['global_data_update'] = callback
 
 	def notify_logged_out(self):
 		if not self.callbacks.get('logged_out'):
@@ -140,10 +142,11 @@ class NSO_API:
 			return
 		self.callbacks['user_data_update'](self, self.context)
 
-	def notify_global_data_update(self):
-		if not self.callbacks.get('global_data_update'):
+	@classmethod
+	def notify_global_data_update(cls):
+		if not cls.global_callbacks.get('global_data_update'):
 			return
-		self.callbacks['global_data_update'](self.global_data)
+		cls.global_callbacks['global_data_update'](cls.global_data)
 
 	def get_user_data(self):
 		keys = {}
@@ -167,16 +170,19 @@ class NSO_API:
 			self.s3.set_keys(keys['games'].get('s3'))
 			self.acnh.set_keys(keys['games'].get('acnh'))
 
-	def get_global_data(self):
-		return self.global_data
+	@classmethod
+	def get_global_data(cls):
+		return cls.global_data
 
-	def load_global_data(self, data):
+	@classmethod
+	def load_global_data(cls, data):
 		if not isinstance(data, dict):
 			return
-		self.global_data = data
+		cls.global_data = data
 
-	def get_global_data_value(self, path):
-		container = self.global_data
+	@classmethod
+	def get_global_data_value(cls, path):
+		container = cls.global_data
 		parts = path.split(".")
 		for p in range(len(parts) - 1):
 			if not parts[p] in container:
@@ -184,15 +190,16 @@ class NSO_API:
 			container = container[parts[p]]
 		return container.get(parts[-1])
 
-	def set_global_data_value(self, path, value):
-		container = self.global_data
+	@classmethod
+	def set_global_data_value(cls, path, value):
+		container = cls.global_data
 		parts = path.split(".")
 		for p in range(len(parts) - 1):
 			if not parts[p] in container:
 				container[parts[p]] = {}
 			container = container[parts[p]]
 		container[parts[-1]] = value
-		self.notify_global_data_update()
+		cls.notify_global_data_update()
 
 	# Discards all keys except for the session_token. This should not be
 	#  needed during normal use, but can be useful for testing.
@@ -406,17 +413,17 @@ class NSO_API:
 
 		prep_req = self.session.prepare_request(req)
 
-		if self.debug >= 2:
+		if self.debug & 0x02:
 			self.dump_http_message(prep_req)
-		elif self.debug >= 1:
+		elif self.debug & 0x01:
 			print(f">> {prep_req.method} {prep_req.url}")
 
 		self.last_activity_time = time.time()
 		res = self.session.send(prep_req)
 
-		if self.debug >= 2:
+		if self.debug & 0x02:
 			self.dump_http_message(res)
-		elif self.debug >= 1:
+		elif self.debug & 0x01:
 			print(f"<< {res.status_code} {res.reason}")
 
 		if not res.status_code in expect_status:
@@ -503,29 +510,33 @@ class NSO_API:
 		self.login = None  # Login complete, so delete login challenge data
 
 		self.session_token = result['session_token']
-		self.user_data_update()
+		self.notify_user_data_update()
 		return True
 
 	# Ensures we have a Nintendo app version.
 	def ensure_app_version(self):
 		app_version = self.get_global_data_value("app_version")
 		if app_version is not None:
+			if self.debug & 0x04:
+				print(f"App version: Cached data: {repr(app_version)}")
 			if time.time() < app_version['expiretime']:
+				if self.debug & 0x04:
+					print(f"App version: Cached version still fresh.")
 				return True
 
-		if self.debug >= 1:
-			print("App version out of date, checking...")
+		if self.debug & 0x04:
+			print("App version: Cached version out of date, checking...")
 
 		version = self.app.get_version()
 		if version is None:
-			if self.debug >= 1:
+			if self.debug & 0x04:
 				print(f"  Failed to get app version, using fallback version {self.FALLBACK_APP_VERSION}")
 			now = int(time.time())
 			expiretime = now + 3600
 			self.set_global_data_value("app_version", {"retrievetime": now, "expiretime": expiretime, "data": {"version": self.FALLBACK_APP_VERSION, "fallback": True}})
 			return False
 
-		if self.debug >= 1:
+		if self.debug & 0x04:
 			print(f"  Found app version: {version}")
 
 		now = int(time.time())
@@ -536,8 +547,12 @@ class NSO_API:
 	def get_app_version(self):
 		app_version = self.get_global_data_value("app_version")
 		if app_version is not None:
+			if self.debug & 0x04:
+				print(f"App version: Using cached version '{app_version['data']['version']}'")
 			return app_version['data']['version']
 
+		if self.debug & 0x04:
+			print(f"App version: Using fallback version '{self.FALLBACK_APP_VERSION}'")
 		return self.FALLBACK_APP_VERSION
 
 	# Ensures we have fresh api_tokens.
